	include findrexx.asm	; include findrexx subroutine
	page
; -------------------------------------------------------------
; This include file contains the CALLREXX variable and two procedures:
;
;	GETINT	gets address of rexx interrupt handler
;	ERRINT	can be used to print error message and exit
;		if GETINT fails
;	LOADCHK checks whether rexx is loaded
;	 input ah = 0 abort if not loaded
;	       ah = 1 return w/carry set if not loaded
;
; Sample usage:
;		call	GETINT	; get addr of rexx interrupt handler
;		jz	continue; if no error, continue
;		jmp	ERRINT	; else print error message and exit
;	continue:
;
; Sample usage for version that gets function code!
;		mov	ah,0	; tell getint to abort if rxintmgr not loaded
;		call	getint	; get rexx interrupt or issue emsg and abort
;		mov	ah,1	; tell loadchk not to abort if rexx not loaded
;		call	loadchk ; check whether rexx is loaded
;		jc	loadrexx; not loaded - free memory and load rexx
;		....
;		pushf		; use these two instructions
;		call	callrexx; to call rexx interrupt
;
; GETINT entry assumption:
;	ES points to the program's PSP (which it will if you are creating
;	a standard COM or EXE program and ES contains its original value
;	from startup).
; GETINT exit:
;	If successful, the zero flag is set and the variable "CALLREXX"
;	contains the addr of the Rexx interrupt handler.
;	Otherwise, AL contains the return code from FINDREXX.
;
; ERRINT entry assumption:
;	AL contains the FINDREXX return code.
; exit conditions:
;	This routine exits to DOS with return code in AL.
;
; NOTE 1: This routine was written without using any constants
; or other good programming stuff so that it can simply be
; included or merged into other programs intact, without
; needing any header files, etc.
;
; NOTE 2: All labels declared in this procedure begin with '$$c'
; to minimize collisions with your own labels.
; -------------------------------------------------------------

;	get address of Rexx interrupt handler
;	-------------------------------------
getint	proc	near
	jmp	$$c_start		; jump around constant
callrexx	dd	?		; address of interrupt handler goes here

;	get rexx interrupt number
$$c_start:
	call	findrexx		; get Rexx interrupt number
	or	al,al			; is interrupt manager installed?
	jnz	$$c_return		; no, exit

;	get address of rexx interrupt handler
	push	es
	mov	al,ah			; get vector number
	mov	ah,35H			; set function (get vector)
	int	21H			; call DOS to get vector in ES:BX
	mov	word ptr callrexx[0],bx ; save IP of Rexx interrupt handler
	mov	word ptr callrexx[2],es ; save CS of Rexx interrupt handler
	pop	es

;	return
$$c_return:
	ret				; return to caller
getint	endp

;	process error finding Rexx interrupt
;	------------------------------------
errint	proc	near
	jmp	$$c_begin		; jump around constants
$$c_er_invalid	db	'Error: Invalid RXINT environment setting',13,10,'$'
$$c_er_inuse	db	'Error: REXX interrupt in use by another program',13,10,'$'
$$c_er_install	db	'Error: RXINTMGR not installed',13,10,'$'
$$c_er_other	db	'Error: could not get REXX interrupt',13,10,'$'

;	pick appropriate message
$$c_begin:
	lea	dx,$$c_er_invalid	; invalid RXINT value
	cmp	al,1
	je	$$c_error_exit
	lea	dx,$$c_er_inuse 	; REXX interrupt in use
	cmp	al,2
	je	$$c_error_exit
	lea	dx,$$c_er_install	; RXINTMGR not installed
	cmp	al,3
	je	$$c_error_exit
	lea	dx,$$c_er_other 	; other FINDREXX error

;	print error message and return to dos
$$c_error_exit:
	push	ax			; save return code
	mov	ah,9			; dos function = display string
	int	21h			; display error message
	pop	ax			; restore return code
	mov	ah,4Ch			; dos function = exit
	int	21h			; return to dos
errint	endp
